from skimage.filters import unsharp_mask

sharpen = unsharp_mask
"""Image sharpen using unsharp masking.

See: `skimage.filters.unsharp_mask() <https://scikit-image.org/docs/stable/api/skimage.filters.html#skimage.filters.unsharp_mask>`_
"""